"use strict";

const t = require("@babel/core").types;

const convertProgramNode = require("./convertProgramNode");

module.exports = function (ast, traverse, code) {
  const state = {
    source: code
  };
  t.VISITOR_KEYS.Property = t.VISITOR_KEYS.ObjectProperty;
  t.VISITOR_KEYS.MethodDefinition = ["key", "value", "decorators", "returnType", "typeParameters"];
  traverse(ast, astTransformVisitor, null, state);
  delete t.VISITOR_KEYS.Property;
  delete t.VISITOR_KEYS.MethodDefinition;
  convertProgramNode(ast);
};

const astTransformVisitor = {
  noScope: true,

  enter(path) {
    const node = path.node;
    node._babelType = node.type;

    if (node.innerComments) {
      delete node.innerComments;
    }

    if (node.trailingComments) {
      delete node.trailingComments;
    }

    if (node.leadingComments) {
      delete node.leadingComments;
    }
  },

  exit(path) {
    const node = path.node;

    if (path.isTypeParameter()) {
      node.type = "Identifier";
      node.typeAnnotation = node.bound;
      delete node.bound;
    }

    if (path.isQualifiedTypeIdentifier()) {
      delete node.id;
    }

    if (path.isObjectTypeProperty()) {
      delete node.key;
    }

    if (path.isObjectTypeIndexer()) {
      delete node.id;
    }

    if (path.isFunctionTypeParam()) {
      delete node.name;
    }

    if (path.isImportDeclaration()) {
      delete node.isType;
    }

    if (path.isTemplateLiteral()) {
      for (let j = 0; j < node.quasis.length; j++) {
        const q = node.quasis[j];
        q.range[0] -= 1;

        if (q.tail) {
          q.range[1] += 1;
        } else {
          q.range[1] += 2;
        }

        q.loc.start.column -= 1;

        if (q.tail) {
          q.loc.end.column += 1;
        } else {
          q.loc.end.column += 2;
        }
      }
    }
  }

};